// use std::collections::HashMap;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct RangeInfo {
    line: u32,
    character: usize,
}

#[derive(Serialize, Deserialize)]
pub struct Range {
    start: RangeInfo,
    end: RangeInfo,
}

#[derive(Serialize, Deserialize)]
pub struct Parameter {
    name: String,
    parameter_type: String,
    init: String,
    range: Range,
}

#[derive(Serialize, Deserialize)]
pub struct Port {
    name: String,
    port_type: String,
    net_type: String,
    width: String,
    range: Range,
}

#[derive(Serialize, Deserialize)]
pub struct Instance {
    name: String,
    instance_type: String,
    instparams: Range,
    instports: Range,
}

#[derive(Serialize, Deserialize)]
pub struct SvlogModule {
    name: String,
    params: Vec<Parameter>,
    ports: Vec<Port>,
    instances: Vec<Instance>,
    range: Range
}

#[derive(Serialize, Deserialize)]
pub struct SvlogInfo {
    pub fast: Vec<SvlogModule>
}

impl SvlogInfo {
    pub fn new_module(&mut self, symbol_name: &str, symbol_line: u32, start_col: usize, end_col: usize) {
        let new_mod = SvlogModule {
            name: symbol_name.to_string(),
            range: Range {
                start: RangeInfo {
                    line: symbol_line,
                    character: start_col
                },
                end: RangeInfo {
                    line: symbol_line,
                    character: end_col
                }
            },
            params: Vec::new(),
            ports: Vec::new(),
            instances: Vec::new()
        };
        self.fast.push(new_mod);
    }

    pub fn add_parameter(&mut self, param_name: &str, param_type: &str, start_line: u32, start_col: usize, end_line: u32, end_col: usize) {
        if let Some(last_mod) = self.fast.last_mut() {
            let parameter = Parameter {
                name: param_name.to_string(),
                parameter_type: param_type.to_string(),
                init: "unknown".to_string(),
                range: Range {
                    start: RangeInfo {
                        line: start_line,
                        character: start_col
                    },
                    end: RangeInfo {
                        line: end_line,
                        character: end_col
                    }
                }
            };
            (*last_mod).params.push(parameter);
        }
    }

    pub fn add_port(&mut self, port_name: &str, port_net_type: &str, port_dir: &str, start_line: u32, start_col: usize, end_line: u32, end_col: usize) {
        if let Some(last_mod) = self.fast.last_mut() {
            let port = Port {
                name: port_name.to_string(),
                port_type: port_dir.to_string(),
                net_type: port_net_type.to_string(),
                width: "unknown".to_string(),
                range: Range {
                    start: RangeInfo {
                        line: start_line,
                        character: start_col
                    },
                    end: RangeInfo {
                        line: end_line,
                        character: end_col
                    }
                }
            };
            (*last_mod).ports.push(port);
        }
    }

    pub fn add_instance(&mut self, symbol_name: &str) {
        if let Some(last_mod) = self.fast.last_mut() {
            let instance = Instance {
                name: symbol_name.to_string(),
                instance_type: (*last_mod.name).to_string(),
                instparams: Range {
                    start: RangeInfo {
                        line: 0,
                        character: 0
                    },
                    end: RangeInfo {
                        line: 0,
                        character: 0
                    }
                },
                instports: Range {
                    start: RangeInfo {
                        line: 0,
                        character: 0
                    },
                    end: RangeInfo {
                        line: 0,
                        character: 0
                    }
                }
            };
            (*last_mod).instances.push(instance);
        }
    }
}

// const RS2JSON_RENAME_LIST: [(&str, &str); 4] = [
//     ("parameter_type", "type"),
//     ("port_type", "type"),
//     ("net_type", "netType"),
//     ("instance_type", "type"),
// ];
